@@ -1,6 +1,6 @@
-/* 
- * Copyright 2007 Secure Endpoints Inc.  
- * 
+/*
+ * Copyright 2007 Secure Endpoints Inc.
+ *
  * All Rights Reserved.
  *
  * This software has been released under the terms of the IBM Public
@@ -148,6 +148,7 @@ int  cm_BPlusDirLookupOriginalName(cm_dirOp_t * op, clientchar_t *entry, fschar_
 long cm_BPlusDirCreateEntry(cm_dirOp_t * op, clientchar_t *entry, cm_fid_t * cfid);
 int  cm_BPlusDirDeleteEntry(cm_dirOp_t * op, clientchar_t *entry);
 long cm_BPlusDirBuildTree(cm_scache_t *scp, cm_user_t *userp, cm_req_t* reqp);
+int  cm_BPlusDirFoo(struct cm_scache *scp, struct cm_dirEntry *dep, void *dummy, osi_hyper_t *entryOffsetp);
 void cm_BPlusDumpStats(void);
 int  cm_MemDumpBPlusStats(FILE *outputFile, char *cookie, int lock);
 
@@ -171,9 +172,10 @@ typedef struct cm_direnum {
     cm_direnum_entry_t 	entry[1];
 } cm_direnum_t;
 
-long cm_BPlusDirEnumerate(cm_scache_t *dscp, cm_user_t *userp, cm_req_t *reqp, 
+long cm_BPlusDirEnumerate(cm_scache_t *dscp, cm_user_t *userp, cm_req_t *reqp,
                           afs_uint32 locked, clientchar_t *maskp, afs_uint32 fetchStatus, cm_direnum_t **enumpp);
 long cm_BPlusDirNextEnumEntry(cm_direnum_t *enump, cm_direnum_entry_t **entrypp);
+long cm_BPlusDirPeekNextEnumEntry(cm_direnum_t *enump, cm_direnum_entry_t **entrypp);
 long cm_BPlusDirFreeEnumeration(cm_direnum_t *enump);
 long cm_BPlusDirEnumTest(cm_scache_t * dscp, cm_user_t *userp, cm_req_t *reqp, afs_uint32 locked);
 long cm_BPlusDirEnumBulkStat(cm_direnum_t *enump);
@@ -188,7 +190,7 @@ extern afs_uint64 bplus_free_time;
 
 
 /************ Accessor Macros *****************************************/
-			
+
 /* low level definition of Nptr value usage */
 #define nAdr(n) (n)->X
 
