@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -30,7 +30,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>    
+#include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -38,6 +38,7 @@
 #include <grp.h>
 #endif
 #include <rx/xdr.h>
+#include <afs/afsutil.h>
 #include "rmtsys.h"
 #include "sys_prototypes.h"
 
@@ -50,7 +51,7 @@ static afs_int32 SetClientCreds(struct clientcred *creds, afs_uint32 * groups);
 int afs_get_pag_from_groups(afs_uint32 g0, afs_uint32 g1);
 void afs_get_groups_from_pag(afs_uint32 pag, afs_uint32 * g0p, afs_uint32 * g1p);
 
-/* Picks up the name of the remote afs client host where the rmtsys 
+/* Picks up the name of the remote afs client host where the rmtsys
  * daemon resides. Since the clients may be diskless and/or readonly
  * ones we felt it's better to rely on an shell environment
  * (AFSSERVER) for the host name first. If that is not set, the
@@ -60,7 +61,7 @@ void afs_get_groups_from_pag(afs_uint32 pag, afs_uint32 * g0p, afs_uint32 * g1p)
 afs_int32
 GetAfsServerAddr(char *syscall)
 {
-    register struct hostent *th;
+    struct hostent *th;
 
     if (hostAddrLookup) {
 	/* Take advantage of caching and assume that the remote host
@@ -73,7 +74,7 @@ GetAfsServerAddr(char *syscall)
     if (!(afs_server = getenv("AFSSERVER"))) {
 	char *home_dir;
 	FILE *fp;
-	int len;
+	int len = 0;
 
 	if (!(home_dir = getenv("HOME"))) {
 	    /* Our last chance is the "/.AFSSERVER" file */
@@ -81,13 +82,15 @@ GetAfsServerAddr(char *syscall)
 	    if (fp == 0) {
 		return 0;
 	    }
-	    fgets(server_name, 128, fp);
-	    fclose(fp);
 	} else {
-	    char pathname[256];
+	    char *pathname;
 
-	    sprintf(pathname, "%s/%s", home_dir, ".AFSSERVER");
+	    afs_asprintf(&pathname, "%s/%s", home_dir, ".AFSSERVER");
+	    if (pathname == NULL)
+		return 0;
 	    fp = fopen(pathname, "r");
+	    free(pathname);
+
 	    if (fp == 0) {
 		/* Our last chance is the "/.AFSSERVER" file */
 		fp = fopen("/.AFSSERVER", "r");
@@ -95,10 +98,10 @@ GetAfsServerAddr(char *syscall)
 		    return 0;
 		}
 	    }
-	    fgets(server_name, 128, fp);
-	    fclose(fp);
 	}
-	len = strlen(server_name);
+	if (fgets(server_name, 128, fp) != NULL)
+	    len = strlen(server_name);
+	fclose(fp);
 	if (len == 0) {
 	    return 0;
 	}
@@ -162,7 +165,7 @@ setpag(void)
     clientcred creds;
     afs_int32 errorcode, errornumber, newpag, ngroups, j;
     afs_uint32 groups[NGROUPS_MAX];
-    
+
     if (!(conn = rx_connection(&errorcode, "setpag"))) {
 	/* Remote call can't be performed for some reason.
 	 * Try the local 'setpag' system call ... */
@@ -242,7 +245,7 @@ pioctl(char *path, afs_int32 cmd, struct ViceIoctl *data, afs_int32 follow)
     inparam_conversion(cmd, InData.rmtbulk_val, 0);
 
     OutData.rmtbulk_len = MAXBUFFERLEN * sizeof(*OutData.rmtbulk_val);
-    OutData.rmtbulk_val = malloc(OutData.rmtbulk_len); 
+    OutData.rmtbulk_val = malloc(OutData.rmtbulk_len);
     if (!OutData.rmtbulk_val) {
 	free(inbuffer);
 	return -1;
@@ -256,8 +259,8 @@ pioctl(char *path, afs_int32 cmd, struct ViceIoctl *data, afs_int32 follow)
 	    /* assuming relative path name */
 	    if (getcwd(pathname, 256) == NULL) {
 		free(inbuffer);
-		printf("getwd failed; exiting\n");
-		exit(1);
+		printf("getwd failed\n");
+		return -1;
 	    }
 	    strcpy(pathname + strlen(pathname), "/");
 	    strcat(pathname, path);
